@@ -4,6 +4,7 @@ use core::cmp::Ordering;
 use core::fmt;
 use core::ops::Add;
 use core::ops::AddAssign;
+use core::ops::Div;
 use core::ops::Index;
 use core::ops::Mul;
 use core::ops::Neg;
@@ -303,6 +304,35 @@ impl<'a, S: Integer> Neg for &'a Vec2d<S> {
     }
 }
 
+impl Mul<Vec2d<i32>> for i32 {
+    type Output = Vec2d<i32>;
+
+    fn mul(self, other: Vec2d<i32>) -> Vec2d<i32> {
+        Vec2d::with(|i| self * other[i])
+    }
+}
+impl<'a> Mul<&'a Vec2d<i32>> for i32 {
+    type Output = Vec2d<i32>;
+
+    fn mul(self, other: &'a Vec2d<i32>) -> Vec2d<i32> {
+        Vec2d::with(|i| self * other[i])
+    }
+}
+impl<'a> Mul<Vec2d<i32>> for &'a i32 {
+    type Output = Vec2d<i32>;
+
+    fn mul(self, other: Vec2d<i32>) -> Vec2d<i32> {
+        Vec2d::with(|i| self * other[i])
+    }
+}
+impl<'a> Mul<&'a Vec2d<i32>> for &'a i32 {
+    type Output = Vec2d<i32>;
+
+    fn mul(self, other: &'a Vec2d<i32>) -> Vec2d<i32> {
+        Vec2d::with(|i| self * other[i])
+    }
+}
+
 impl Mul<Vec2d<i64>> for i64 {
     type Output = Vec2d<i64>;
 
@@ -310,7 +340,6 @@ impl Mul<Vec2d<i64>> for i64 {
         Vec2d::with(|i| self * other[i])
     }
 }
-
 impl<'a> Mul<&'a Vec2d<i64>> for i64 {
     type Output = Vec2d<i64>;
 
@@ -318,7 +347,6 @@ impl<'a> Mul<&'a Vec2d<i64>> for i64 {
         Vec2d::with(|i| self * other[i])
     }
 }
-
 impl<'a> Mul<Vec2d<i64>> for &'a i64 {
     type Output = Vec2d<i64>;
 
@@ -326,7 +354,6 @@ impl<'a> Mul<Vec2d<i64>> for &'a i64 {
         Vec2d::with(|i| self * other[i])
     }
 }
-
 impl<'a> Mul<&'a Vec2d<i64>> for &'a i64 {
     type Output = Vec2d<i64>;
 
@@ -367,6 +394,35 @@ impl<'a, S: Integer> Mul<&'a Vec2d<S>> for &'a Vec2d<S> {
     }
 }
 
+impl<S: Integer> Div<S> for Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn div(self, other: S) -> Vec2d<S> {
+        Vec2d::with(|i| self[i] / other)
+    }
+}
+impl<'a, S: Integer> Div<&'a S> for Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn div(self, other: &'a S) -> Vec2d<S> {
+        Vec2d::with(|i| self[i] / other)
+    }
+}
+impl<'a, S: Integer> Div<S> for &'a Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn div(self, other: S) -> Vec2d<S> {
+        Vec2d::with(|i| self[i] / other)
+    }
+}
+impl<'a, S: Integer> Div<&'a S> for &'a Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn div(self, other: &'a S) -> Vec2d<S> {
+        Vec2d::with(|i| self[i] / other)
+    }
+}
+
 impl<S: Integer> AddAssign for Vec2d<S> {
     fn add_assign(&mut self, other: Vec2d<S>) {
         *self = Vec2d::with(|i| self[i] + other[i])
@@ -548,8 +604,16 @@ mod tests {
     }
 
     #[test]
-    fn test_mul_sv() {
-        let v = v2d(3, 7);
+    fn test_mul_sv_32() {
+        let v: Vec2d<i32> = v2d(3, 7);
+        assert_eq!(v2d(6, 14), 2 * v);
+        assert_eq!(v2d(6, 14), 2 * &v);
+        assert_eq!(v2d(6, 14), &2 * v);
+        assert_eq!(v2d(6, 14), &2 * &v);
+    }
+    #[test]
+    fn test_mul_sv_64() {
+        let v: Vec2d<i64> = v2d(3, 7);
         assert_eq!(v2d(6, 14), 2 * v);
         assert_eq!(v2d(6, 14), 2 * &v);
         assert_eq!(v2d(6, 14), &2 * v);
@@ -566,6 +630,15 @@ mod tests {
         assert_eq!(13, &u * &v);
     }
 
+    #[test]
+    fn test_div_vs() {
+        let v = v2d(6, 14);
+        assert_eq!(v2d(3, 7), v / 2);
+        assert_eq!(v2d(3, 7), &v / 2);
+        assert_eq!(v2d(3, 7), v / &2);
+        assert_eq!(v2d(3, 7), &v / &2);
+    }
+
     #[test]
     fn test_add_assign() {
         let mut u = v2d(2, 1);
@@ -597,4 +670,20 @@ mod tests {
         let v = v2d(3, 7);
         assert_eq!(v2d(3, 7), u.max(v));
     }
+
+    #[test]
+    fn test_is_zero_true() {
+        let v = v2d(0, 0);
+        assert!(v.is_zero());
+    }
+    #[test]
+    fn test_is_zero_false_x() {
+        let v = v2d(1, 0);
+        assert!(!v.is_zero());
+    }
+    #[test]
+    fn test_is_zero_false_y() {
+        let v = v2d(0, 1);
+        assert!(!v.is_zero());
+    }
 }
