@@ -4,6 +4,7 @@ use core::cmp::Ordering;
 use core::fmt;
 use core::ops::Add;
 use core::ops::AddAssign;
+use core::ops::Div;
 use core::ops::Index;
 use core::ops::Mul;
 use core::ops::Neg;
@@ -323,6 +324,35 @@ impl<'a, S: Integer> Neg for &'a Vec3d<S> {
     }
 }
 
+impl Mul<Vec3d<i32>> for i32 {
+    type Output = Vec3d<i32>;
+
+    fn mul(self, other: Vec3d<i32>) -> Vec3d<i32> {
+        Vec3d::with(|i| self * other[i])
+    }
+}
+impl<'a> Mul<&'a Vec3d<i32>> for i32 {
+    type Output = Vec3d<i32>;
+
+    fn mul(self, other: &'a Vec3d<i32>) -> Vec3d<i32> {
+        Vec3d::with(|i| self * other[i])
+    }
+}
+impl<'a> Mul<Vec3d<i32>> for &'a i32 {
+    type Output = Vec3d<i32>;
+
+    fn mul(self, other: Vec3d<i32>) -> Vec3d<i32> {
+        Vec3d::with(|i| self * other[i])
+    }
+}
+impl<'a> Mul<&'a Vec3d<i32>> for &'a i32 {
+    type Output = Vec3d<i32>;
+
+    fn mul(self, other: &'a Vec3d<i32>) -> Vec3d<i32> {
+        Vec3d::with(|i| self * other[i])
+    }
+}
+
 impl Mul<Vec3d<i64>> for i64 {
     type Output = Vec3d<i64>;
 
@@ -330,7 +360,6 @@ impl Mul<Vec3d<i64>> for i64 {
         Vec3d::with(|i| self * other[i])
     }
 }
-
 impl<'a> Mul<&'a Vec3d<i64>> for i64 {
     type Output = Vec3d<i64>;
 
@@ -338,7 +367,6 @@ impl<'a> Mul<&'a Vec3d<i64>> for i64 {
         Vec3d::with(|i| self * other[i])
     }
 }
-
 impl<'a> Mul<Vec3d<i64>> for &'a i64 {
     type Output = Vec3d<i64>;
 
@@ -346,7 +374,6 @@ impl<'a> Mul<Vec3d<i64>> for &'a i64 {
         Vec3d::with(|i| self * other[i])
     }
 }
-
 impl<'a> Mul<&'a Vec3d<i64>> for &'a i64 {
     type Output = Vec3d<i64>;
 
@@ -387,6 +414,35 @@ impl<'a, S: Integer> Mul<&'a Vec3d<S>> for &'a Vec3d<S> {
     }
 }
 
+impl<S: Integer> Div<S> for Vec3d<S> {
+    type Output = Vec3d<S>;
+
+    fn div(self, other: S) -> Vec3d<S> {
+        Vec3d::with(|i| self[i] / other)
+    }
+}
+impl<'a, S: Integer> Div<&'a S> for Vec3d<S> {
+    type Output = Vec3d<S>;
+
+    fn div(self, other: &'a S) -> Vec3d<S> {
+        Vec3d::with(|i| self[i] / other)
+    }
+}
+impl<'a, S: Integer> Div<S> for &'a Vec3d<S> {
+    type Output = Vec3d<S>;
+
+    fn div(self, other: S) -> Vec3d<S> {
+        Vec3d::with(|i| self[i] / other)
+    }
+}
+impl<'a, S: Integer> Div<&'a S> for &'a Vec3d<S> {
+    type Output = Vec3d<S>;
+
+    fn div(self, other: &'a S) -> Vec3d<S> {
+        Vec3d::with(|i| self[i] / other)
+    }
+}
+
 impl<S: Integer> AddAssign for Vec3d<S> {
     fn add_assign(&mut self, other: Vec3d<S>) {
         *self = Vec3d::with(|i| self[i] + other[i])
@@ -582,8 +638,16 @@ mod tests {
     }
 
     #[test]
-    fn test_mul_sv() {
-        let v = v3d(3, 7, 1);
+    fn test_mul_sv_32() {
+        let v: Vec3d<i32> = v3d(3, 7, 1);
+        assert_eq!(v3d(6, 14, 2), 2 * v);
+        assert_eq!(v3d(6, 14, 2), 2 * &v);
+        assert_eq!(v3d(6, 14, 2), &2 * v);
+        assert_eq!(v3d(6, 14, 2), &2 * &v);
+    }
+    #[test]
+    fn test_mul_sv_64() {
+        let v: Vec3d<i64> = v3d(3, 7, 1);
         assert_eq!(v3d(6, 14, 2), 2 * v);
         assert_eq!(v3d(6, 14, 2), 2 * &v);
         assert_eq!(v3d(6, 14, 2), &2 * v);
@@ -600,6 +664,15 @@ mod tests {
         assert_eq!(18, &u * &v);
     }
 
+    #[test]
+    fn test_div_vs() {
+        let v = v3d(6, 14, 2);
+        assert_eq!(v3d(3, 7, 1), v / 2);
+        assert_eq!(v3d(3, 7, 1), &v / 2);
+        assert_eq!(v3d(3, 7, 1), v / &2);
+        assert_eq!(v3d(3, 7, 1), &v / &2);
+    }
+
     #[test]
     fn test_add_assign() {
         let mut u = v3d(2, 1, 5);
@@ -631,4 +704,25 @@ mod tests {
         let v = v3d(3, 7, 1);
         assert_eq!(v3d(3, 7, 5), u.max(v));
     }
+
+    #[test]
+    fn test_is_zero_true() {
+        let v = v3d(0, 0, 0);
+        assert!(v.is_zero());
+    }
+    #[test]
+    fn test_is_zero_false_x() {
+        let v = v3d(1, 0, 0);
+        assert!(!v.is_zero());
+    }
+    #[test]
+    fn test_is_zero_false_y() {
+        let v = v3d(0, 1, 0);
+        assert!(!v.is_zero());
+    }
+    #[test]
+    fn test_is_zero_false_z() {
+        let v = v3d(0, 0, 1);
+        assert!(!v.is_zero());
+    }
 }
