@@ -4,6 +4,7 @@ use core::cmp::Ordering;
 use core::fmt;
 use core::ops::Add;
 use core::ops::AddAssign;
+use core::ops::Div;
 use core::ops::Index;
 use core::ops::Mul;
 use core::ops::Neg;
@@ -372,6 +373,36 @@ impl<'a, S: Integer> Neg for &'a Vec4d<S> {
         Vec4d::with(|i| -self[i])
     }
 }
+
+impl Mul<Vec4d<i32>> for i32 {
+    type Output = Vec4d<i32>;
+
+    fn mul(self, other: Vec4d<i32>) -> Vec4d<i32> {
+        Vec4d::with(|i| self * other[i])
+    }
+}
+impl<'a> Mul<&'a Vec4d<i32>> for i32 {
+    type Output = Vec4d<i32>;
+
+    fn mul(self, other: &'a Vec4d<i32>) -> Vec4d<i32> {
+        Vec4d::with(|i| self * other[i])
+    }
+}
+impl<'a> Mul<Vec4d<i32>> for &'a i32 {
+    type Output = Vec4d<i32>;
+
+    fn mul(self, other: Vec4d<i32>) -> Vec4d<i32> {
+        Vec4d::with(|i| self * other[i])
+    }
+}
+impl<'a> Mul<&'a Vec4d<i32>> for &'a i32 {
+    type Output = Vec4d<i32>;
+
+    fn mul(self, other: &'a Vec4d<i32>) -> Vec4d<i32> {
+        Vec4d::with(|i| self * other[i])
+    }
+}
+
 impl Mul<Vec4d<i64>> for i64 {
     type Output = Vec4d<i64>;
 
@@ -379,7 +410,6 @@ impl Mul<Vec4d<i64>> for i64 {
         Vec4d::with(|i| self * other[i])
     }
 }
-
 impl<'a> Mul<&'a Vec4d<i64>> for i64 {
     type Output = Vec4d<i64>;
 
@@ -387,7 +417,6 @@ impl<'a> Mul<&'a Vec4d<i64>> for i64 {
         Vec4d::with(|i| self * other[i])
     }
 }
-
 impl<'a> Mul<Vec4d<i64>> for &'a i64 {
     type Output = Vec4d<i64>;
 
@@ -395,7 +424,6 @@ impl<'a> Mul<Vec4d<i64>> for &'a i64 {
         Vec4d::with(|i| self * other[i])
     }
 }
-
 impl<'a> Mul<&'a Vec4d<i64>> for &'a i64 {
     type Output = Vec4d<i64>;
 
@@ -436,6 +464,35 @@ impl<'a, S: Integer> Mul<&'a Vec4d<S>> for &'a Vec4d<S> {
     }
 }
 
+impl<S: Integer> Div<S> for Vec4d<S> {
+    type Output = Vec4d<S>;
+
+    fn div(self, other: S) -> Vec4d<S> {
+        Vec4d::with(|i| self[i] / other)
+    }
+}
+impl<'a, S: Integer> Div<&'a S> for Vec4d<S> {
+    type Output = Vec4d<S>;
+
+    fn div(self, other: &'a S) -> Vec4d<S> {
+        Vec4d::with(|i| self[i] / other)
+    }
+}
+impl<'a, S: Integer> Div<S> for &'a Vec4d<S> {
+    type Output = Vec4d<S>;
+
+    fn div(self, other: S) -> Vec4d<S> {
+        Vec4d::with(|i| self[i] / other)
+    }
+}
+impl<'a, S: Integer> Div<&'a S> for &'a Vec4d<S> {
+    type Output = Vec4d<S>;
+
+    fn div(self, other: &'a S) -> Vec4d<S> {
+        Vec4d::with(|i| self[i] / other)
+    }
+}
+
 impl<S: Integer> AddAssign for Vec4d<S> {
     fn add_assign(&mut self, other: Vec4d<S>) {
         *self = Vec4d::with(|i| self[i] + other[i])
@@ -688,8 +745,16 @@ mod tests {
     }
 
     #[test]
-    fn test_mul_sv() {
-        let v = v4d(3, 7, 1, 4);
+    fn test_mul_sv_32() {
+        let v: Vec4d<i32> = v4d(3, 7, 1, 4);
+        assert_eq!(v4d(6, 14, 2, 8), 2 * v);
+        assert_eq!(v4d(6, 14, 2, 8), 2 * &v);
+        assert_eq!(v4d(6, 14, 2, 8), &2 * v);
+        assert_eq!(v4d(6, 14, 2, 8), &2 * &v);
+    }
+    #[test]
+    fn test_mul_sv_64() {
+        let v: Vec4d<i64> = v4d(3, 7, 1, 4);
         assert_eq!(v4d(6, 14, 2, 8), 2 * v);
         assert_eq!(v4d(6, 14, 2, 8), 2 * &v);
         assert_eq!(v4d(6, 14, 2, 8), &2 * v);
@@ -706,6 +771,15 @@ mod tests {
         assert_eq!(2, &u * &v);
     }
 
+    #[test]
+    fn test_div_vs() {
+        let v = v4d(6, 14, 2, 8);
+        assert_eq!(v4d(3, 7, 1, 4), v / 2);
+        assert_eq!(v4d(3, 7, 1, 4), &v / 2);
+        assert_eq!(v4d(3, 7, 1, 4), v / &2);
+        assert_eq!(v4d(3, 7, 1, 4), &v / &2);
+    }
+
     #[test]
     fn test_add_assign() {
         let mut u = v4d(2, 1, 5, 4);
@@ -737,4 +811,30 @@ mod tests {
         let v = v4d(3, 7, 1, 4);
         assert_eq!(v4d(3, 7, 5, 4), u.max(v));
     }
+
+    #[test]
+    fn test_is_zero_true() {
+        let v = v4d(0, 0, 0, 0);
+        assert!(v.is_zero());
+    }
+    #[test]
+    fn test_is_zero_false_x() {
+        let v = v4d(1, 0, 0, 0);
+        assert!(!v.is_zero());
+    }
+    #[test]
+    fn test_is_zero_false_y() {
+        let v = v4d(0, 1, 0, 0);
+        assert!(!v.is_zero());
+    }
+    #[test]
+    fn test_is_zero_false_z() {
+        let v = v4d(0, 0, 1, 0);
+        assert!(!v.is_zero());
+    }
+    #[test]
+    fn test_is_zero_false_w() {
+        let v = v4d(0, 0, 0, 1);
+        assert!(!v.is_zero());
+    }
 }
