@@ -28,6 +28,8 @@ where
     Self: PartialOrd,
     Self: VectorOps<S, Self>,
     Self: for<'a> VectorOps<S, &'a Self>,
+    Self: ops::Div<S, Output = Self>,
+    Self: for<'a> ops::Div<&'a S, Output = Self>,
 {
     /// The dimension of the vectors in this type.
     const DIM: usize;
@@ -53,6 +55,11 @@ where
         Self::with(|_| S::zero())
     }
 
+    /// Returns `true` if a vector is the zero vector.
+    fn is_zero(&self) -> bool {
+        self == &Self::zero()
+    }
+
     /// Apply min by component
     fn min(&self, other: Self) -> Self {
         Self::with(|i| self[i].min(other[i]))
